var RetractionDestructForm = function()
{
	var rd = {};
	var patientInfo;
	var caseIsValid = false;
	var labSentDateIsValid = false;
	var labReceivedDateIsValid = false;
	var bloodSentDateIsValid = false;
	var bloodReceivedDateIsValid = false;
	var frozenSentDateIsValid = false;
	var frozenReceivedDateIsValid = false;
	var forwardedSentDateIsValid = false;
	var rccSydSentDateIsValid = false;
	var debug = 0;
	var TRUNCATE_SIZE = [-1, 30, 20, 15];
	
	rd.initPage = function()
	{
		caseIsValid = false;
		labSentDateIsValid = false;
		labReceivedDateIsValid = false;
		bloodSentDateIsValid = false;
		bloodReceivedDateIsValid = false;
		frozenSentDateIsValid = false;
		frozenReceivedDateIsValid = false;
		forwardedSentDateIsValid = false;
		rccSydSentDateIsValid = false;
	
		previewTitle = Doc.element('previewTitle');
		previewList = Doc.element('previewList');
		
		// Step 1
		Events.addEventHandler('step-1', 'wizard-validate', rd.validateStep1);
		Events.addEventHandler('caseName', 'blur', rd.caseNameOnChange);
		Events.addEventHandler('caseName', 'keypress', rd.keyPressCheck);
		//Events.addEventHandler('caseName', 'keypress', Wizard.goNextOnTabOrEnter);
		
		// Step 2
		Events.addEventHandler('step-2', 'wizard-initialize', rd.initializeStep2);
		Events.addEventHandler('step-2', 'wizard-validate', rd.validateStep2);

		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);		

		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', rd.submit);
		
		Doc.show('step-1');
		Doc.show('gonext');
	}

	/**
		Check case name if key pressed if Tab or Return/Enter.
	*/
	rd.keyPressCheck = function(event)
	{
		// Check case name if key pressed if Tab or Return/Enter
		if (event.keyCode == 9 || event.keyCode == 13)
		{
			rd.caseNameOnChange(event);
		}
	}
	
	/**
		Check that the case number is valid.
	*/
	rd.caseNameOnChange = function(event)
	{
		var frm = document.forms['reggie'];
		var caseName = frm.caseName.value.toUpperCase();
		caseIsValid = false;
		if (caseName == '')
		{
			Wizard.setInputStatus('caseName', 'invalid', 'Missing');
			frm.caseName.focus();
			return;
		}
		else if (!Reggie.isValidCaseName(caseName, false)) // 'C' suffix is not used
		{
			rd.validatePersonalNumber(caseName);
			if (pnrIsValid)
			{
				caseIsValid = true;
			}
			else
			{
				// Entered number is neither a valid case name or personal number
				// Try to give as appropriate error message as possible
				if (caseName.length < 7 || pnrErrorMsg == null || pnrErrorMsg == '')
				{
					// The entered number is probably a faulty case name
					Wizard.setInputStatus('caseName', 'invalid', 'Only 7-digits name is allowed .');
				}
				else
				{
					Wizard.setInputStatus('caseName', 'invalid', pnrErrorMsg);
				}
				frm.caseName.focus();
				return;
			}
		}

		frm.caseName.value = caseName;

		// Check that the patient item is of type "Retract" or "RetroNo"		
		var url = '../Retraction.servlet?ID='+App.getSessionId();
		url += '&cmd=GetCaseInfo';
		url += '&caseName=' + encodeURIComponent(caseName);
		Wizard.asyncJsonRequest(url, rd.onCaseInfoLoaded);
	}

	rd.onCaseInfoLoaded = function(response)
	{
		var frm = document.forms['reggie'];
		
		// Get biosource information from the AJAX response
		caseInfo = response.caseInfo;
		patientInfo = response.patient;

		if (patientInfo)
		{
			var patientName = patientInfo.name;
			var retractStatus = patientInfo.retractStatus;
			// Check if patient name does not start with "Retract" or "RetroNo"
			if (patientName.indexOf('Retract') != 0 && patientName.indexOf('RetroNo') != 0)
			{
				Wizard.setInputStatus('caseName', 'invalid', 'Patient name \"' + patientName + '\" does not start with \"Retract\" or \"RetroNo\".');
				return;
			}
			else if (retractStatus == null || retractStatus != 'ReadyForDestruction')
			{
				Wizard.setInputStatus('caseName', 'invalid', 'Patient name \"' + patientName + '\" has \"RetractStatus\" annotation value set to \"'+retractStatus+'\", not \"ReadyForDestruction\".');
				return;
			}
			else
			{
				caseIsValid = true;
				Wizard.setInputStatus('caseName', 'valid');
				Wizard.goNext(true);
				return;
			}
		}
	}
	
	/**
		Check that the case number is valid.
	*/
	rd.validateStep1 = function(event)
	{
		if (event.detail.auto)
		{
			if (!caseIsValid) 
			{
				event.preventDefault();
			}
		}
		else
		{
			if (!caseIsValid) event.preventDefault();
		}
	}

	rd.validatePersonalNumber = function(pnr)
	{
		pnrIsValid = false;
		pnrIsAcceptable = true;
		pnrErrorMsg = null;
		
		var pnrLen = pnr.length;
		if (pnrLen != 12 && pnrLen != 10)
		{
			Wizard.setInputStatus('caseName', 'warning', pnrLen < 12 ? 'Personal number too short' : 'Personal number too long');
			if (pnrLen < 12)
			{
				pnrErrorMsg = 'Personal number too short';
			}
			else
			{
				pnrErrorMsg = 'Personal number too long';
			}
			return;
		}

		if (!pnr.match(/^\d+$/))
		{
			if (!pnr.match(/^\d+[A-Za-z]$/))
			{
				return;
			}
		}
		
		var datePart = pnr.substring(0, pnrLen - 4); // Remove last four digits
		var dateFormat = pnrLen == 10 ? 'yyMMdd' : 'yyyyMMdd';
		var isValidDate = Dates.isDate(datePart, dateFormat);
		if (!isValidDate)
		{
			var dayInMonth = parseInt(datePart.substring(-2), 10); // Last two characters
			if (dayInMonth > 60)
			{
				// 'Samordningsnummer' has day-in-month + 60; check this
				dayInMonth -= 60;
				var tmpPnr = pnr.substring(0, pnrLen - 6); // First 4 or 6 digits is year+month
				if (dayInMonth < 10) tmpPnr += '0';
				tmpPnr += dayInMonth;
				isValidDate = Dates.isDate(tmpPnr, dateFormat);
			}
		}
		if (!isValidDate)
		{
			Wizard.setInputStatus('caseName', 'warning', 'Personal number: Not a valid date');
			pnrErrorMsg = 'Personal number: Not a valid date';
			return;
		}
		
		if (!pnr.match(/^\d+[A-Za-z]$/))
		{
			if (!Reggie.personalNumberControlDigitCheck(pnr.substr(pnrLen == 10 ? 0 : 2)))
			{
				Wizard.setInputStatus('caseName', 'warning', 'Personal number: Invalid control digit');
				pnrErrorMsg = 'Personal number: Invalid control digit';
				return;
			}
		}

		Wizard.setInputStatus('caseName', 'valid');
		pnrIsValid = true;
	}

	/**
		Load information about the given case.
	*/
	rd.initializeStep2 = function(event)
	{
		var frm = document.forms['reggie'];
		var caseName = frm.caseName.value;

		var url = '../Retraction.servlet?ID='+App.getSessionId();
		url += '&cmd=GetCaseInfoBeforeDestruction';
		url += '&caseName=' + encodeURIComponent(caseName);
		
		Wizard.showLoadingAnimation('Loading case information...');
		Wizard.asyncJsonRequest(url, rd.caseInfoLoaded);
	}

	/**
		Initalize the second step based on the information we have about the case.
	*/
	rd.caseInfoLoaded = function(response)
	{
		var frm = document.forms['reggie'];
		var caseName = frm.caseName.value;
		patientInfo = response.patient;
		var caseInfo = response.caseInfo;

		// Initialize biosource retraction request and response dates
		var destructRetract = false;
		if (patientInfo)
		{
			var patientName = patientInfo.name;
			// Check if patient name starts with "Retract" or "RetroNo"
			if (patientName.indexOf('Retract') == 0 || patientName.indexOf('RetroNo') == 0)
			{
				destructRetract = true;
				var registrationDate = patientInfo.registrationDate;
				frm.patientName.value = patientName;
				frm.patientRegistrationDate.value = registrationDate;
				// Retraction request and response dates
				frm.labSentDate.value = patientInfo.labSentDate;
				frm.labReceivedDate.value = patientInfo.labReceivedDate;
				frm.bloodSentDate.value = patientInfo.bloodSentDate;
				frm.bloodReceivedDate.value = patientInfo.bloodReceivedDate;
				frm.frozenSentDate.value = patientInfo.frozenSentDate;
				frm.frozenReceivedDate.value = patientInfo.frozenReceivedDate;
				frm.forwardedSentDate.value = patientInfo.forwardedSentDate;
				frm.rccSydSentDate.value = patientInfo.rccSydSentDate;

				rd.addRetractTable2(response);
			}
		}
		frm.patientName.disabled = true;
		frm.patientRegistrationDate.disabled = true;
		frm.labSentDate.disabled = true;
		frm.labReceivedDate.disabled = true;
		frm.bloodSentDate.disabled = true;
		frm.bloodReceivedDate.disabled = true;
		frm.frozenSentDate.disabled = true;
		frm.frozenReceivedDate.disabled = true;
		frm.forwardedSentDate.disabled = true;
		frm.rccSydSentDate.disabled = true;

		Wizard.setCurrentStep(2);

		Doc.show('step-1');
		if (destructRetract)
		{
			Doc.hide('gonext');
			Doc.show('gocancel');
			Doc.show('goregister');
		}
	}

	rd.addRetractTable2 = function(response)
	{
		var frm = document.forms['reggie'];
		var numCases = response.numCases;
		var caseName = frm.caseName.value;
		//var site = response.site;
		patientInfo = response.patient;
		var caseInfo = response.caseInfo;
		var specimenInfo = response.specimen;
		var comment = '<b>To be deleted</b>';

		var caseArray = [];
		if (numCases > 0)
		{
			caseArray.push(response.caseName1);
		}
		if (numCases > 1)
		{
			caseArray.push(response.caseName2);
		}

		var html = '<table id="retractionItemsTable2">';

		// Patient Header row
		html += '<thead class="bg-filled-100">';
		html += '<tr>';
		html += '<th colspan="3">Patient</th>';
		html += '</tr>';
		html += '<tr>';
		html += '<th>type</th>';
		html += '<th>name</th>';
		html += '<th>comment</th>';
		html += '</tr>';
		html += '</thead>';

		html += '<tbody>';
		
		// Patient
		var patientInfo = response.patient;
		if (patientInfo)
		{
			html += '<tr>';
			html += '<td>' + patientInfo.subtype + '</td>';
			html += '<td>' + patientInfo.name + '</td>';
			html += '<td><b>To be replaced by a copy with the same name</b></td>';
			html += '</tr>';
		}
		html += '</tbody>';

		// Case
		var caseInfo = response.caseInfo;
		if (caseInfo && caseInfo.length > 0)
		{
			// Case Header row
			html += '<thead class="bg-filled-100">';
			html += '<tr>';
			html += '<th colspan="3">Cases</th>';
			html += '</tr>';
			html += '<tr>';
			html += '<th>type</th>';
			html += '<th>name</th>';
			html += '<th>comment</th>';
			html += '</tr>';
			html += '</thead>';

			html += '<tbody>';

			var truncateAt = TRUNCATE_SIZE[Math.min(caseInfo.length-1, TRUNCATE_SIZE.length)];
			for (var i = 0; i < caseInfo.length; i++)
			{
				var c = caseInfo[i];
				var flag = c.flag ? c.flag : '-';
				var site = response.site;
				var comment = c.comment ? c.comment : '-';
				//var autoProcessing = s.autoProcessing ? s.autoProcessing : '-';
				html += '<tr>';
				html += '<td>Case</td>';
				html += '<td>' + c.name + '</td>';
				html += '<td><b>To be replaced by a copy with the same name</b></td>';
				html += '</tr>';
			}
			html += '</tbody>';
		}
		else
		{
			html += '<thead class="bg-filled-100">';
			html += '<tr id="case.name2">';
			html += '<th id="case-header2">Case</th>';
			html += '<td>' + rd.asNoInfo('No case information has been registered') + '</td>';
			html += '</tr>';
			html += '</thead>';
		}

		// Blood
		var blood = response.blood;
		if (blood && blood.length > 0)
		{
			// Blood Sample Header row
			html += '<thead class="bg-filled-100">';
			html += '<tr>';
			html += '<th colspan="3">Blood samples</th>';
			html += '</tr>';
			html += '<tr>';
			html += '<th>type</th>';
			html += '<th>name</th>';
			html += '<th>comment</th>';
			html += '</tr>';
			html += '</thead>';

			html += '<tbody>';
		
			var truncateAt = TRUNCATE_SIZE[Math.min(blood.length-1, TRUNCATE_SIZE.length)];
			var comment = '<b>To be deleted</d>';
			if (caseInfo && caseInfo.length == 0)
			{
				comment = '<b>To be replaced by a case copy with the same SCAN-B ID</d>';
			}
			for (var i = 0; i < blood.length; i++)
			{
				var b = blood[i];
				html += '<tr>';
				html += '<td>Blood</td>';
				html += '<td>' + b.name + '</td>';
				html += '<td>' + comment + '</td>';
				html += '</tr>';
			}
			html += '</tbody>';
		}
		else
		{
			// Blood Sample Header row
			html += '<thead class="bg-filled-100">';
			html += '<tr id="blood.name2">';
			html += '<th id="blood-header2">Blood</th>';
			html += '<td>' + rd.asNoInfo('No blood information has been registered') + '</td>';
			html += '</tr>';
			html += '</thead>';
		}

		// Other sample items
		var subItems = response.subItems;

		// Check if any items exist
		var itemExists = false;
		if (subItems && subItems.length > 0)
		{
			itemExists = true;
		}

		if (itemExists)
		{
			// Sample Items Header row
			html += '<thead class="bg-filled-100">';
			html += '<tr>';
			html += '<th colspan="3">Other items</th>';
			html += '</tr>';
			html += '<tr>';
			html += '<th class="dottedleft">type</th>';
			html += '<th>name</th>';
			html += '<th>comment</th>';
			html += '</tr>';
			html += '</thead>';
		}
		else
		{
			// Other Items Header row
			html += '<thead class="bg-filled-100">';
			html += '<tr id="other.name2">';
			html += '<th id="other-header2">Other sample items</th>';
			html += '<td>' + rd.asNoInfo('No other sample item information has been registered') + '</td>';
			html += '</tr>';
			html += '</thead>';
		}
		html += '<tbody>';

		// Other sample item rows
		html += htmlTableRowsForOtherItems(subItems);
		html += '</tbody>';
		html += '</table>';
		Doc.element('retractTable2').innerHTML = html;
	}

	rd.asCaseList = function(allCases, mainCase)
	{
		var html = '';
		for (var i = 0; i < allCases.length; i++)
		{
			var cse = allCases[i];
			if (html != '') html += ', ';
			if (cse != mainCase)
			{
				//html += '<span class="link linked-case" data-case-name="'+cse+'" title="Show summary of case #'+cse+'">'+cse+'</span>';
				html += cse;
			}
			else
			{
				html += '<b>' + cse + '</b>';
			}
		}
		return allCases.length > 0 ? html : null;
	}

	// Format value as a date
	rd.asDate = function(value)
	{
		if (!value) return '';
		if (value.length == 8)
		{
			value = value.substr(0, 4) + '-' + value.substr(4, 2) + '-' + value.substr(6, 2);
		}
		return value;
	}

	// Format as date+time value
	rd.asDateTime = function(value, compareToDate)
	{
		if (!value) return '';
		if (value.length == 8)
		{
			value = rd.asDate(value);
		}
		else if (value.length == 13)
		{
			// If the compareToDate is the same day as the 'value' date, skip the date and replace with white-space
			if (compareToDate && value.substr(0, 8) == compareToDate.substr(0, 8))
			{
				value = '<span class="invisible">'+rd.asDate(value.substr(0, 8)) + '</span> ' + value.substr(9, 2) + ':'+value.substr(11, 2)
			}
			else
			{
				value = rd.asDate(value.substr(0, 8)) + ' ' + value.substr(9, 2) + ':'+value.substr(11, 2);
			}
		}
		return value;
	}

	// Style the message as a warning message
	rd.asWarning = function(message)
	{
		message = '<span class="warning">' + message + '</span>';
		return message;
	}

	// Generate a 'Missing' warning if the value is missing
	rd.warnIfMissing = function(value)
	{
		var message = value ? value : rd.asWarning('Missing');
		return message;
	}
	
	// Style the message as a 'no information' message
	rd.asNoInfo = function(message)
	{
		message = '<span class="no-info">' + message + '</span>';
		return message;
	}
	
	rd.asConsentAnyDate = function(consent)
	{
		var warn = !consent;
		var message;
		if (!consent)
		{
			message = 'Missing';
		}
		else
		{
			message = consent;
			
		}
		return warn ? rd.asWarning(message) : message;
	}
	
	// A biowell as plate name
	rd.asBioPlateName = function(well)
	{
		if (!well) return '';
		var plate = well.bioPlate;
		if (!plate) return '';
		var text = plate.name;
		return text;
	}

	// A biowell as the location
	rd.asPureBioPlateLocation = function(well)
	{
		if (!well) return '';
		var plate = well.bioPlate;
		var text = well.location;
		if (plate.storage)
		{
			var storage = plate.storage;
			var tmp = [];
			if (storage.name) tmp[tmp.length] = storage.name;
			if (storage.section) tmp[tmp.length] = 'section: ' +storage.section;
			if (storage.tray) tmp[tmp.length] = 'tray: ' +storage.tray;
			if (storage.position) tmp[tmp.length] = 'position: ' +storage.position;

			//text = '<span class="more-info" title="' + tmp.join('; ')+'">' + text + ' </span>';
			text = text + ' ' + tmp.join('; ');
		}
		return text;
	}

	rd.truncate = function(value, maxLength)
	{
		if (!value) return value;
		if (maxLength > 2 && value.length > maxLength) 
		{
			var tmp = '<span class="truncated" title="'+value+'">'+value.substring(0, maxLength-2) + '...</span>';
			value = tmp;
		}
		return value;
	}

	function htmlTableRowsForOtherItems(itemJsonArr)
	{
		// Table item row
		var html = '';
		if (itemJsonArr && itemJsonArr.length > 0)
		{
			var truncateAt = TRUNCATE_SIZE[Math.min(itemJsonArr.length-1, TRUNCATE_SIZE.length)];
			for (var i = 0; i < itemJsonArr.length; i++)
			{
				var item = itemJsonArr[i];
				var name = item.name;
				var type = item.type;
				var comment = item.comment;
				if (comment && comment == 'To be deleted')
				{
					comment = '<b>' + comment + '</b>';
				}
				html += '<tr>';
				html += '<td>' + type + '</td>';
				html += '<td>' + name + '</td>';
				html += '<td>' + comment + '</td>';
				html += '</tr>';
			}
		}
		return html;
	}


	function htmlTableRowsForItems(
		itemJsonArr,
		itemType,
		itemTypeVarName,
		dateType,
		otherDataType,
		storageType,
		setRetracted,
		setAutoProcessingDisabled)
	{
		// Table item row
		var html = '';
		if (itemJsonArr && itemJsonArr.length > 0)
		{
			var truncateAt = TRUNCATE_SIZE[Math.min(itemJsonArr.length-1, TRUNCATE_SIZE.length)];
			for (var i = 0; i < itemJsonArr.length; i++)
			{
				var item = itemJsonArr[i];
				var flag = item.flag ? item.flag : '-';
				var autoProcessing = item.autoProcessing ? item.autoProcessing : '-';
				var itemDate = '-';
				if (dateType)
				{
					if (dateType == 'Sampling')
					{
						itemDate = rd.warnIfMissing(rd.asDateTime(item.samplingDate));
					}
					else if (dateType == 'Partition')
					{
						itemDate = rd.warnIfMissing(rd.asDateTime(item.partitionDate));
					}
					else if (dateType == 'Lysis')
					{
						itemDate = rd.warnIfMissing(rd.asDateTime(item.lysisDate));
					}
				}
				else
				{
					dateType = '-';
				}
				var otherDateName = '-';
				var otherDateValue = '-';
				var otherDataName = '-';
				var otherDataValue = '-';
				if (otherDataType)
				{
					if (otherDataType == 'PAD')
					{
						otherDataName = 'PAD';
						otherDataValue = rd.warnIfMissing(item.pad);
					}
					else if (otherDataType == 'nofSlides')
					{
						otherDataName = 'No. slides';
						otherDataValue = item.numStains ? item.numStains : '-';
					}
					else if (otherDataType == 'nofQc')
					{
						otherDataName = 'No. QC';
						otherDataValue = item.nofQc ? item.nofQc : '-';
					}
					else if (otherDataType == 'nofDilutes')
					{
						otherDataName = 'No. Dilutes';
						var dilItems = item.dilItems;
						otherDataValue = dilItems ? dilItems.length : 0;
					}
				}
				var storageName = '-';
				var storageId = '-';
				var storagePosition = '-';
				if (storageType)
				{
					if (storageType == 'NA')
					{
						storageName = 'NA';
						storageId = 'NA';
						storagePosition = 'NA';
					}
					else
					{
						storageName = storageType;
						storageId = rd.asBioPlateName(item.bioWell);
						storagePosition = rd.asPureBioPlateLocation(item.bioWell);
					}
				}
				var comment = '<b>To be deleted</b>';
				if (itemType && itemType == 'Pooled Library')
				{
					comment = 'No change';
				}
				html += '<tr>';
				html += '<td>' + itemType + '</td>';
				html += '<td>' + item.name + '</td>';
				html += '<td>' + comment + '</td>';
				html += '</tr>';
			}
		}
		return html;
	}


	function htmlTableRowsForSubItems(
		itemJsonArr,
		subItemType,
		subItemTypeVarName,
		dateType,
		otherDataType,
		storageType,
		setRetracted,
		setAutoProcessingDisabled)
	{
		// Table item row
		var html = '';
		if (itemJsonArr && itemJsonArr.length > 0)
		{
			var truncateAt = TRUNCATE_SIZE[Math.min(itemJsonArr.length-1, TRUNCATE_SIZE.length)];
			for (var i = 0; i < itemJsonArr.length; i++)
			{
				var item = itemJsonArr[i];
				var subItemJsonArr = null;
				if (subItemType == 'Stained')
				{
					subItemJsonArr = item.stainedItems;
				}
				else if (subItemType == 'RNA-QC')
				{
					subItemJsonArr = item.rnaQcItems;
				}
				else if (subItemType == 'Library dilute')
				{
					subItemJsonArr = item.dilItems;
				}
				if (subItemJsonArr)
				{
					html += htmlTableRowsForItems(subItemJsonArr, subItemType, subItemTypeVarName, dateType, otherDataType, storageType, setRetracted, setAutoProcessingDisabled);
				}
			}
		}
		return html;
	}


	function checkValueForOtherSampleItems(
		varName,
		selectedSamples)
	{
		// Other sample items
		var specimen = response.specimen;
		var noSpecimen = response.noSpecimen;
		var histology = response.histology;
		var lysate = response.lysate;
		var rna = response.rna;
		var dna = response.dna;
		var ft = response.flowThrough;
		var mrna = response.mrna;
		var cdna = response.cdna;
		var lib = response.lib;
		var pooledLib = response.pooledlib;

		selectedSamples = checkValueForItems(specimen, 'Specimen', 'specimen', varName, selectedSamples);
		selectedSamples = checkValueForItems(noSpecimen, 'NoSpecimen', 'nospecimen', varName, selectedSamples);
		selectedSamples = checkValueForItems(histology, 'Histology', 'histology', varName, selectedSamples);
		selectedSamples = checkValueForSubItems(histology, 'Stained', 'stained', varName, selectedSamples);
		selectedSamples = checkValueForItems(lysate, 'Lysate', 'lysate', varName, selectedSamples);
		selectedSamples = checkValueForItems(rna, 'RNA', 'rna', varName, selectedSamples);
		selectedSamples = checkValueForSubItems(rna, 'RNA-QC', 'rnaqc', varName, selectedSamples);
		selectedSamples = checkValueForItems(dna, 'DNA', 'dna', varName, selectedSamples);
		selectedSamples = checkValueForItems(ft, 'FlowThrough', 'flowthrough', varName, selectedSamples);
		selectedSamples = checkValueForItems(mrna, 'mRNA', 'mrna', varName, selectedSamples);
		selectedSamples = checkValueForItems(cdna, 'cDNA', 'cdna', varName, selectedSamples);
		selectedSamples = checkValueForItems(lib, 'Library', 'library', varName, selectedSamples);
		selectedSamples = checkValueForSubItems(lib, 'Library dilute', 'librarydilute', varName, selectedSamples);
		selectedSamples = checkValueForItems(pooledLib, 'Pooled Library', 'pooledlibrary', varName, selectedSamples);
		return selectedSamples;
	}

	rd.labDateOnChange = function()
	{
		Wizard.setInputStatus('labDate');
		var frm = document.forms['reggie'];		

		labSentDateIsValid = false;
		var labSentDate = frm.labSentDate.value;
		labSentDate = rd.dateCheck(labSentDate, 'lab sent', 'labDate');		
		if (labSentDate != null)
		{
			frm.labSentDate.value = labSentDate;
			labSentDateIsValid = true;
		}

		labReceivedDateIsValid = false;
		var labReceivedDate = frm.labReceivedDate.value;
		labReceivedDate = rd.dateCheck(labReceivedDate, 'lab received', 'labDate');		
		if (labReceivedDate != null)
		{
			frm.labReceivedDate.value = labReceivedDate;
			labReceivedDateIsValid = true;
		}

		if (labSentDateIsValid && labReceivedDateIsValid)
		{
			var result = rd.causalityDateCheck(labSentDate, 'lab sent', labReceivedDate, 'lab received', 'labDate');
			if (result != null)
			{
				Wizard.setInputStatus('labDate', 'valid');
			}
		}
	}

	rd.bloodDateOnChange = function()
	{
		Wizard.setInputStatus('bloodDate');
		var frm = document.forms['reggie'];		

		bloodSentDateIsValid = false;
		var bloodSentDate = frm.bloodSentDate.value;
		bloodSentDate = rd.dateCheck(bloodSentDate, 'blood sent', 'bloodDate');		
		if (bloodSentDate != null)
		{
			frm.bloodSentDate.value = bloodSentDate;
			bloodSentDateIsValid = true;
		}

		bloodReceivedDateIsValid = false;
		var bloodReceivedDate = frm.bloodReceivedDate.value;
		bloodReceivedDate = rd.dateCheck(bloodReceivedDate, 'blood received', 'bloodDate');		
		if (bloodReceivedDate != null)
		{
			frm.bloodReceivedDate.value = bloodReceivedDate;
			bloodReceivedDateIsValid = true;
		}

		if (bloodSentDateIsValid && bloodReceivedDateIsValid)
		{
			var result = rd.causalityDateCheck(bloodSentDate, 'blood sent', bloodReceivedDate, 'blood received', 'bloodDate');
			if (result != null)
			{
				Wizard.setInputStatus('bloodDate', 'valid');
			}
		}
	}

	rd.frozenDateOnChange = function()
	{
		Wizard.setInputStatus('frozenDate');
		var frm = document.forms['reggie'];		

		frozenSentDateIsValid = false;
		var frozenSentDate = frm.frozenSentDate.value;
		frozenSentDate = rd.dateCheck(frozenSentDate, 'frozen sent', 'frozenDate');		
		if (frozenSentDate != null)
		{
			frm.frozenSentDate.value = frozenSentDate;
			frozenSentDateIsValid = true;
		}

		frozenReceivedDateIsValid = false;
		var frozenReceivedDate = frm.frozenReceivedDate.value;
		frozenReceivedDate = rd.dateCheck(frozenReceivedDate, 'frozen received', 'frozenDate');		
		if (frozenReceivedDate != null)
		{
			frm.frozenReceivedDate.value = frozenReceivedDate;
			frozenReceivedDateIsValid = true;
		}

		if (frozenSentDateIsValid && frozenReceivedDateIsValid)
		{
			var result = rd.causalityDateCheck(frozenSentDate, 'frozen sent', frozenReceivedDate, 'frozen received', 'frozenDate');
			if (result != null)
			{
				Wizard.setInputStatus('frozenDate', 'valid');
			}
		}
	}

	rd.forwardedDateOnChange = function()
	{
		Wizard.setInputStatus('forwardedDate');
		var frm = document.forms['reggie'];		

		forwardedSentDateIsValid = false;
		var forwardedSentDate = frm.forwardedSentDate.value;
		forwardedSentDate = rd.dateCheck(forwardedSentDate, 'forwarded sent', 'forwardedDate');		
		if (forwardedSentDate != null)
		{
			frm.forwardedSentDate.value = forwardedSentDate;
			forwardedSentDateIsValid = true;
		}

		if (forwardedSentDateIsValid)
		{
			Wizard.setInputStatus('forwardedDate', 'valid');
		}
	}

	rd.rccSydDateOnChange = function()
	{
		Wizard.setInputStatus('rccSydDate');
		var frm = document.forms['reggie'];		

		rccSydSentDateIsValid = false;
		var rccSydSentDate = frm.rccSydSentDate.value;
		rccSydSentDate = rd.dateCheck(rccSydSentDate, 'RCC Syd sent', 'rccSydDate');		
		if (rccSydSentDate != null)
		{
			frm.rccSydSentDate.value = rccSydSentDate;
			rccSydSentDateIsValid = true;
		}

		if (rccSydSentDateIsValid)
		{
			Wizard.setInputStatus('rccSydDate', 'valid');
		}
	}

	rd.dateCheck = function(dateValue, dateName, dateMessageKey)
	{
		if (dateValue != '')
		{
			// Auto-fill the date if it's only given with 4(MMdd) or 6(yyMMdd) digits.
			dateValue = Reggie.autoFillDate(dateValue, true);

			var d = Dates.parseString(dateValue, 'yyyyMMdd');
			if (d == null)
			{
				Wizard.setInputStatus(dateMessageKey, 'invalid', 'Not a valid '+dateName+' date');
				return null;
			}
			
			var today = new Date();
			if (d.getTime() > today.getTime())
			{
				Wizard.setInputStatus(dateMessageKey, 'invalid', 'Future '+dateName+' date is not valid');
				return null;
			}			
		}
		return dateValue;
	}

	rd.causalityDateCheck = function(date1, dateName1, date2, dateName2, dateMessageKey)
	{
		if (date1 != null && date1 != '' && date2 != null && date2 != '')
		{
			// Auto-fill the date if it's only given with 4(MMdd) or 6(yyMMdd) digits.
			date1 = Reggie.autoFillDate(date1, true);
			date2 = Reggie.autoFillDate(date2, true);

			var d1 = Dates.parseString(date1, 'yyyyMMdd');
			if (d1 == null)
			{
				Wizard.setInputStatus(dateMessageKey, 'invalid', 'Not a valid '+dateName1+' date');
				return null;
			}
			
			var d2 = Dates.parseString(date2, 'yyyyMMdd');
			if (d2 == null)
			{
				Wizard.setInputStatus(dateMessageKey, 'invalid', 'Not a valid '+dateName2+' date');
				return null;
			}
			
			if (d1.getTime() > d2.getTime())
			{
				Wizard.setInputStatus(dateMessageKey, 'invalid', 'Causality violation: '+dateName1+' date is later than '+dateName2+' date');
				return null;
			}	
		}
		return date1;
	}

	rd.validateStep2 = function(event)
	{
		// Get up-to-date values for date validation variables
		rd.labDateOnChange();
		rd.bloodDateOnChange();
		rd.frozenDateOnChange();
		rd.forwardedDateOnChange();
		rd.rccSydDateOnChange();

		// Step 2 is valid if all date values are valid ('null' value is allowed)
		var datesAreValid = false;
		if (labSentDateIsValid &&
			labReceivedDateIsValid &&
			bloodSentDateIsValid &&
			bloodReceivedDateIsValid &&
			frozenSentDateIsValid &&
			frozenReceivedDateIsValid &&
			forwardedSentDateIsValid &&
			rccSydSentDateIsValid)
		{
			datesAreValid = true;
		}
		
		if (!datesAreValid) event.preventDefault();
	}
	
	// Style the message as a warning message
	rd.asWarning = function(message)
	{
		message = '<span class="warning">' + message + '</span>';
		return message;
	}

	// Generate a 'Missing' warning if the value is missing
	rd.warnIfMissing = function(value)
	{
		var message = value ? value : rd.asWarning('Missing');
		return message;
	}
	
	rd.goPrint = function()
	{
		var caseName = Data.get('page-data', 'case-name');
		var printNote = '<b>Note!</b> For better printing set page orientation to <i>portrait</i>.<br>';
		printNote += ' You may have to <i>scale down</i> to fit everything on the width of the page.';
		Reggie.openPrintWindow('all-content', 'Case summary - ' + Strings.encodeTags(caseName), 'portrait', printNote, '../', 'case_summary.css');
	}

	rd.submit = function()
	{
		var frm = document.forms['reggie'];
		var caseName = frm.caseName.value;

		var retractionInfo = {};
		retractionInfo.caseName = caseName;

		var submitInfo = {};
		submitInfo.retractionInfo = retractionInfo;

		var url = '../Retraction.servlet?ID=' + App.getSessionId();
		url += '&cmd=RegisterDestruction';

		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, rd.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
	
	rd.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}

	return rd;
}();

Doc.onLoad(RetractionDestructForm.initPage);

